@@ -1,7 +1,7 @@
 package Module::Patch;
 
-our $DATE = '2014-12-05'; # DATE
-our $VERSION = '0.20'; # VERSION
+our $DATE = '2015-04-01'; # DATE
+our $VERSION = '0.21'; # VERSION
 
 use 5.010001;
 use strict;
@@ -13,7 +13,6 @@ use Carp;
 use Module::Load;
 use Module::Loaded;
 use Monkey::Patch::Action qw();
-use Scalar::Util qw(reftype);
 use SHARYANTO::Array::Util qw(match_array_or_regex);
 use Package::MoreUtil qw(list_package_contents package_exists);
 
@@ -71,7 +70,7 @@ sub import {
             } elsif ($v == 2) {
                 $mpv = "0.07-0.09";
             }
-            croak "$self ".(${$self."::VERSION"} // "?").
+            croak "$self ".( ${"$self\::VERSION" } // "?" ).
                 " requires Module::Patch $mpv (patch_data format v=$v),".
                     " this is Module::Patch ".($Module::Patch::VERSION // '?').
                         " (v=$curv), please install an older version of ".
@@ -120,7 +119,8 @@ sub import {
         }
 
         ${"$self\::handles"} = patch_package(
-            $target, $pdata->{patches}, {force=>$force});
+            $target, $pdata->{patches},
+            {force=>$force, patch_module=>ref($self) || $self});
     }
 }
 
@@ -157,7 +157,7 @@ sub patch_package {
         my @target_subs;
         my %tp = list_package_contents($target);
         for (keys %tp) {
-            if ((reftype($tp{$_}) // '') eq 'CODE' && !/^\*/) {
+            if (ref($tp{$_}) eq 'CODE' && !/^\*/) {
                 push @target_subs, $_;
             }
         }
@@ -166,14 +166,16 @@ sub patch_package {
       PATCH:
         for my $pspec (@$patches_spec) {
             my $act = $pspec->{action};
-            $act or die "BUG: patch[$i]: no action supplied";
+            my $errp = ($opts->{patch_module} ? "$opts->{patch_module}: ":"").
+                "patch[$i]"; # error prefix
+            $act or die "BUG: $errp: no action supplied";
             $act =~ /\A(wrap|add|replace|add_or_replace|delete)\z/ or die
-                "BUG: patch[$i]: action '$pspec->{action}' unknown";
+                "BUG: $errp: action '$pspec->{action}' unknown";
             if ($act eq 'delete') {
-                $pspec->{code} and die "BUG: patch[$i]: for action 'delete', ".
+                $pspec->{code} and die "BUG: $errp: for action 'delete', ".
                     "code must not be supplied";
             } else {
-                $pspec->{code} or die "BUG: patch[$i]: code not supplied";
+                $pspec->{code} or die "BUG: $errp: code not supplied";
             }
 
             my $sub_names = ref($pspec->{sub_name}) eq 'ARRAY' ?
@@ -182,7 +184,7 @@ sub patch_package {
                 $_ = qr/.*/    if $_ eq ':all';
                 $_ = qr/^_/    if $_ eq ':private';
                 $_ = qr/^[^_]/ if $_ eq ':public';
-                die "BUG: patch[$i]: unknown tag in sub_name $_" if /^:/;
+                die "BUG: $errp: unknown tag in sub_name $_" if /^:/;
             }
 
             my @s;
@@ -206,14 +208,14 @@ sub patch_package {
                     [@$mod_versions] : [$mod_versions];
                 for (@$mod_versions) {
                     $_ = qr/.*/    if $_ eq ':all';
-                    die "BUG: patch[$i]: unknown tag in mod_version $_"
+                    die "BUG: $errp: unknown tag in mod_version $_"
                         if /^:/;
                 }
 
                 my $ver_match=match_array_or_regex(
                     $target_version, $mod_versions);
                 unless ($ver_match) {
-                    carp "patch[$i]: Target module version $target_version ".
+                    carp "$errp: Target module version $target_version ".
                         "does not match [".join(", ", @$mod_versions)."], ".
                             ($opts->{force} ?
                                  "patching anyway (force)":"skipped"). ".";
@@ -249,7 +251,7 @@ Module::Patch - Patch package with a set of patches
 
 =head1 VERSION
 
-This document describes version 0.20 of Module::Patch (from Perl distribution Module-Patch), released on 2014-12-05.
+This document describes version 0.21 of Module::Patch (from Perl distribution Module-Patch), released on 2015-04-01.
 
 =head1 SYNOPSIS
 
@@ -483,7 +485,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by perlancar@cpan.org.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
